package com.logicbig.examples.json.employee;

import com.logicbig.cli.annotation.Argument;
import com.logicbig.cli.annotation.Command;
import com.logicbig.cli.annotation.Option;
import com.logicbig.cli.shell.QuickCLIShell;
import com.logicbig.examples.json.employee.service.EmployeeService;

public class EmployeeCommandHandler {

    private EmployeeService service = ApplicationContext.getInstance().getEmployeeService();

    @Command(name = "del", desc = "Deletes employee by id")
    private void deleteEmployee(@Argument(name = "id", desc = "The id of the employee to be deleted") long id) {
        String s = service.deleteEmployee(id);
        String result = "";
        if (s != null) {
            QuickCLIShell.getWriter().printLine("Employee deleted with id=" + id);
            QuickCLIShell.getWriter().printLine(s);
            return;
        }
        QuickCLIShell.getWriter().printLine("Employee is not deleted. id=" + id);
    }

    @Command(name = "create", desc = "Creates the new employee record")
    private String createEmployee(
            @Option(name = "name", desc = "The employee name", mandatory = true) String name,
            @Option(name = "dept", desc = "The department name", valuesAllowed = {"HR", "IT", "ADMIN"}) String dept,
            @Option(name = "phone", desc = "Employee phone number", mandatory = false) String phone,
            @Option(name = "cell", desc = "Employee cell phone number", mandatory = true) String cellPhone) {
        String employee = service.createEmployee(name, dept, phone, cellPhone);
        if (employee != null) {
            return "Employee created: \n" + employee;
        }
        return "Employee not created";
    }

    @Command(name = "list",
            desc = "Lists employees based on the provided criteria. If no criteria provided are employees all listed")
    private String listEmployee(@Argument(name = "criteria",
            desc = "key:value pair. Examples: (1)\"name:Jane Mayo\" (2)\"phones/cell:435-323-3333\"")
                                String criteria) {

        if (criteria != null && !criteria.contains(":")) {
            return "Error: criteria should be key:value format e.g. \"name:Jane Mayo\"";
        }
        String result;
        if (criteria != null) {
            String[] split = criteria.split(":");
            long i = -1;
            try {
                i = Long.parseLong(split[1]);
            } catch (NumberFormatException e) {
                return "Error: the id value should be a number";
            }

            result = service.getEmployeeList(split[0], i);
        } else {
            result = service.getEmployeeList(null, null);
        }
        return result == null || result.isEmpty() ? "No record found" : result;
    }
}