package com.logicbig.examples.json.employee.service;

import org.fluttercode.datafactory.impl.DataFactory;

import javax.json.*;
import java.io.StringWriter;
import java.util.*;

public class InMemoryJsonEmployeeService implements EmployeeService {
    private JsonArray employeeArray;
    private JsonWriterFactory writerFactory;


    public InMemoryJsonEmployeeService() {
        Map<String, Object> properties = new HashMap<>(1);
        properties.put("javax.json.stream.JsonGenerator.prettyPrinting", Boolean.valueOf(true));
        writerFactory = Json.createWriterFactory(properties);

        DataFactory dataFactory = new DataFactory();

        JsonArrayBuilder personsArrayBuilder = Json.createArrayBuilder();

        for (int i = 0; i < 5; i++) {
            JsonObject personObject = createEmployeeObject(
                    i + 1
                    , dataFactory.getName()
                    , dataFactory.getItem(Arrays.asList("HR", "IT", "ADMIN"))
                    , generatePhoneNumber(dataFactory)
                    , generatePhoneNumber(dataFactory));

            personsArrayBuilder.add(personObject);
        }

        employeeArray = personsArrayBuilder.build();
    }

    private JsonObject createEmployeeObject(int id, String employeeName, String dept, String phone, String cellPhone) {
        JsonObject personObject = Json.createObjectBuilder()
                .add("id", id)
                .add("name", employeeName)
                .add("dept", dept)
                .add("phones", createPhoneArrayObject(phone, cellPhone))
                .build();
        return personObject;
    }

    private JsonArray createPhoneArrayObject(String phone, String cellPhone) {
        return Json.createArrayBuilder()
                .add(phone == null ? Json.createObjectBuilder().add("phone", JsonValue.NULL) :
                                     Json.createObjectBuilder().add("phone", phone))
                .add(Json.createObjectBuilder()
                        .add("cell", cellPhone).build()).build();
    }

    private String generatePhoneNumber(DataFactory dataFactory) {
        return dataFactory.getNumberText(3) + "-" +
                dataFactory.getNumberText(3) + "-" +
                dataFactory.getNumberText(4);
    }

    @Override
    public String getEmployeeList(String keyPath, Object value) {

        return keyPath == null ? asString(employeeArray) : asString(searchEmployee(keyPath, value));
    }

    private String asString(JsonStructure structure) {
        if (structure != null) {
            StringWriter stringWriter = new StringWriter();
            writerFactory.createWriter(stringWriter).write(structure);
            return stringWriter.toString();
        }
        return null;
    }

    @Override
    public String deleteEmployee(long id) {
        JsonArray array = searchEmployee("id", id);
        if (array != null && array.size() == 1) {
            JsonArrayBuilder newArrayBuilder = Json.createArrayBuilder();
            for (JsonValue jsonValue : employeeArray) {
                if (jsonValue instanceof JsonObject) {
                    JsonNumber employeeId = ((JsonObject) jsonValue).getJsonNumber("id");
                    if (id == employeeId.longValue()) {
                        continue;
                    } else {
                        newArrayBuilder.add(jsonValue);
                    }
                }
            }
            employeeArray = newArrayBuilder.build();
            return asString((JsonStructure) array.get(0));
        }

        return null;
    }


    @Override
    public String createEmployee(String employeeName, String dept, String phone, String cellPhone) {
        JsonObject employeeObject = createEmployeeObject(employeeArray.size() + 1, employeeName, dept, phone, cellPhone);
        JsonArrayBuilder arrayBuilder = Json.createArrayBuilder();
        employeeArray.forEach(obj -> arrayBuilder.add(obj));
        arrayBuilder.add(employeeObject);
        employeeArray = arrayBuilder.build();
        return asString(employeeObject);
    }


    private JsonArray searchEmployee(String keyPath, Object value) {

        LinkedHashSet<JsonObject> foundObjects = new LinkedHashSet<>();
        if (search(employeeArray, keyPath, value, null, foundObjects)) {
            JsonArrayBuilder arrayBuilder = Json.createArrayBuilder();
            for (JsonObject foundObject : foundObjects) {
                arrayBuilder.add(foundObject);
            }
            return arrayBuilder.build();
        }
        return null;
    }

    private boolean search(final JsonValue jsonValue, final String keyPath,
                           final Object value, String currentPath, Set<JsonObject> foundObjects) {

        switch (jsonValue.getValueType()) {
            case OBJECT:
                JsonObject object = (JsonObject) jsonValue;
                for (Map.Entry<String, JsonValue> entry : object.entrySet()) {
                    String path = currentPath == null ? entry.getKey() : currentPath + "/" + entry.getKey();
                    if (path.equals(keyPath)) {
                        if (search(entry.getValue(), keyPath, value, path, foundObjects)) {
                            if (currentPath == null) {
                                foundObjects.add(object);
                            }
                            return true;
                        }
                    } else if (keyPath.startsWith(path)) {
                        if (search(entry.getValue(), keyPath, value, path, foundObjects)) {
                            if (currentPath == null) {
                                foundObjects.add(object);
                            }
                            return true;
                        }
                    }
                }
                break;
            case ARRAY:
                boolean result = false;
                for (JsonValue jsonArrayElement : (JsonArray) jsonValue) {
                    result = search(jsonArrayElement, keyPath, value, currentPath, foundObjects) || result;
                }
                return result;
            case STRING:
                JsonString jsonString = (JsonString) jsonValue;
                return value.equals(jsonString.getString());
            case NUMBER:
                JsonNumber jsonNumber = (JsonNumber) jsonValue;
                if (value instanceof Number) {
                    return ((Number) value).doubleValue() == jsonNumber.doubleValue();

                }
                return false;
            case NULL:
                return value == null;
            case TRUE:
                return value.equals(true);
            case FALSE:
                return value.equals(false);
            default:
                if (currentPath.equals(keyPath) && value.equals(jsonValue.toString())) {
                    return true;
                }


        }
        return false;
    }
}